/**
 * Dream
 * Copyright (C) 2003 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: matthieu.leclercq@inrialpes.fr
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Map;

import org.objectweb.dream.PushException;
import org.objectweb.dream.PushPushDreamComponent;
import org.objectweb.dream.message.ChunkAlreadyExistsException;
import org.objectweb.dream.message.ExtensibleMessage;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This message transformer adds a {@link IPChannelDestinationChunk }containing
 * the inet address and the listenning port of a channel. <br>
 * The name of the added chunk is by default
 * {@link IPChannelDestinationChunk#DEFAULT_NAME}, it can be changed using
 * {@link AddIPChannelAttributeController#setChunkName(String)}.<br>
 * The inet address can be specified using the
 * {@link AddIPChannelAttributeController#setInetAddress(String) }method. If
 * not specified, the local address is used.
 */
public class AddIPChannelDestinationChunkImpl extends PushPushDreamComponent
    implements
      AddIPChannelAttributeController
{

  /** the message manager client interface of this component. */
  protected MessageManager messageManagerItf;

  /**
   * the name of the <code>IPChannelDestinationChunk</code> added by this
   * transformer.
   */
  protected String         chunkName = IPChannelDestinationChunk.DEFAULT_NAME;

  /** the inet address set in the chunk. */
  protected InetAddress    inetAddress;

  /** the port set in the chunk. */
  protected int            port;

  /**
   * @see org.objectweb.dream.Push#push(Message, Map)
   */
  public void push(Message message, Map context) throws PushException
  {
    IPChannelDestinationChunk chunk = (IPChannelDestinationChunk) message
        .getChunk(chunkName);
    if (chunk == null)
    {
      if (message instanceof ExtensibleMessage)
      {
        chunk = (IPChannelDestinationChunk) messageManagerItf
            .createChunk(IPChannelDestinationChunk.TYPE);
        try
        {
          ((ExtensibleMessage) message).addChunk(chunkName,
              IPChannelDestinationChunk.TYPE, chunk);
        }
        catch (ChunkAlreadyExistsException e)
        {
          // can't happend
        }
      }
      else
      {
        logger.log(BasicLevel.ERROR, "Invalid message : not extensible");
        throw new InternalError("Invalid message : not extensible");
      }
    }
    chunk.setChannelDestinationAddr(inetAddress);
    chunk.setChannelDestinationPort(port);
    outPushItf.push(message, context);
  }

  // ---------------------------------------------------------------------------
  // Implementation of AddIPChannelAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see AddIPChannelAttributeController#getChunkName()
   */
  public String getChunkName()
  {
    return chunkName;
  }

  /**
   * @see AddIPChannelAttributeController#setChunkName(String)
   */
  public void setChunkName(String chunkName)
  {
    this.chunkName = chunkName;
  }

  /**
   * @see AddIPChannelAttributeController#getInetAddress()
   */
  public String getInetAddress()
  {
    return inetAddress.getCanonicalHostName();
  }

  /**
   * @see AddIPChannelAttributeController#setInetAddress(String)
   */
  public void setInetAddress(String inetAddress) throws UnknownHostException
  {
    this.inetAddress = InetAddress.getByName(inetAddress);
  }

  /**
   * @see AddIPChannelAttributeController#getPort()
   */
  public int getPort()
  {
    return port;
  }

  /**
   * @see AddIPChannelAttributeController#setPort(int)
   */
  public void setPort(int port)
  {
    this.port = port;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{OUT_PUSH_ITF_NAME, MessageManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  // ---------------------------------------------------------------------------
  // Overriding of LifeCycleController methods
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    super.startFc();
    if (inetAddress == null)
    {
      try
      {
        inetAddress = InetAddress.getLocalHost();
      }
      catch (UnknownHostException e)
      {
        throw new IllegalLifeCycleException(
            "Unable to get local inet address : " + e.getLocalizedMessage());
      }
    }
  }
}