/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.message.Message;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Socket manager managing only one open socket at a time. Socket is retrieved
 * using a {@link IPSocketManager }, the IP and port value are set with the
 * {@link SocketManagerIPStaticAttributeController }attribute controller
 * interface.
 */
public class SocketManagerIPStaticImpl extends AbstractComponent
    implements
      SocketManager,
      SocketManagerIPStaticAttributeController
{

  protected boolean         destinationChanged = false;
  protected boolean         inUse              = false;
  protected SocketState     currentSocket;

  // ---------------------------------------------------------------------------
  // Attribute fields
  // ---------------------------------------------------------------------------

  protected InetAddress     destinationAddress;
  protected int             destinationPort;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  protected IPSocketManager delegateSocketManagerItf;

  // ---------------------------------------------------------------------------
  // Implementation of the SocketManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see SocketManager#getSocket(Message)
   */
  public synchronized SocketState getSocket(Message message)
      throws IOException, InterruptedException
  {
    if (inUse)
    {
      throw new IOException("Socket is already in use.");
    }
    inUse = true;
    if (currentSocket == null || currentSocket.isClosed() || destinationChanged)
    {
      if (currentSocket != null)
      {
        delegateSocketManagerItf.releaseSocket(currentSocket, false);
      }
      currentSocket = delegateSocketManagerItf.getSocket(destinationAddress,
          destinationPort);
      destinationChanged = false;
    }
    return currentSocket;
  }

  /**
   * @see SocketManager#releaseSocket(SocketState, boolean)
   */
  public synchronized void releaseSocket(SocketState socketState, boolean error)
  {
    if (socketState != currentSocket)
    {
      throw new IllegalArgumentException(
          "The given SocketState is not the last returned SocketState");
    }
    if (destinationChanged)
    {
      delegateSocketManagerItf.releaseSocket(socketState, error);
      currentSocket = null;
      destinationChanged = false;
    }
    inUse = false;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the AttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see SocketManagerIPStaticAttributeController#getDestinationHostname()
   */
  public String getDestinationHostname()
  {
    return destinationAddress.getCanonicalHostName();
  }

  /**
   * @see SocketManagerIPStaticAttributeController#setDestinationHostname(String)
   */
  public void setDestinationHostname(String hostname)
      throws UnknownHostException
  {
    destinationAddress = InetAddress.getByName(hostname);
    destinationChanged = true;
  }

  /**
   * @see SocketManagerIPStaticAttributeController#getDestinationPort()
   */
  public int getDestinationPort()
  {
    return destinationPort;
  }

  /**
   * @see SocketManagerIPStaticAttributeController#setDestinationPort(int)
   */
  public void setDestinationPort(int port)
  {
    if (destinationPort != port)
    {
      destinationPort = port;
      destinationChanged = true;
    }
  }

  // ---------------------------------------------------------------------------
  // Overriden method of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
   */
  public void stopFc() throws IllegalLifeCycleException
  {
    synchronized (this)
    {
      currentSocket.close();
      currentSocket = null;
    }
    super.stopFc();
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{IPSocketManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(IPSocketManager.ITF_NAME))
    {
      delegateSocketManagerItf = (IPSocketManager) serverItf;
    }
  }
}