/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.utobcast;

import org.objectweb.dream.protocol.Process;

/**
 * This interface defines methods for managing a group of processes.
 */
public interface ProcessMembership
{
  /** The commonly used name to refer to this interface. */
  String ITF_NAME = "process-membership";

  /**
   * Returns the leader process.
   * 
   * @return the leader process.
   * @throws InterruptedException if the component is interrupted while waiting
   *           for a leader to be elected.
   */
  Process getLeader() throws InterruptedException;

  /**
   * Sets the leader process.
   * 
   * @param leader the leader to set.
   */
  void setLeader(Process leader);

  /**
   * Returns the backup process.
   * 
   * @return the backup process.
   * @throws InterruptedException if the component is interrupted while waiting
   *           for a backup to be elected.
   */
  Process getBackup() throws InterruptedException;

  /**
   * Sets the backup process.
   * 
   * @param backup the backup to set.
   */
  void setBackup(Process backup);

  /**
   * Returns the process to which this process membership component belongs.
   * 
   * @return the process to which this process membership component belongs.
   * @throws InterruptedException if the component is interrupted while waiting
   *           for knowing its identity.
   */
  Process getMyself() throws InterruptedException;

  /**
   * Sets the process to which this process membership component belongs.
   * 
   * @param myself the process to set.
   */
  void setMyself(Process myself);

  /**
   * Returns an array containing all the processes except backup and leader
   * processes.
   * 
   * @return all the processes except backup and leader processes.
   */
  Process[] getOtherProcesses();

  /**
   * Sets the processes to be considered for broadcast (except backup and
   * leader).
   * 
   * @param processes all the processes except backup and leader processes.
   */
  void setOtherProcesses(Process[] processes);

  /**
   * Adds a process to the group of processes.
   * 
   * @param process the process to be added.
   */
  void addProcess(Process process);

  /**
   * Removes a process from the group of processes.
   * 
   * @param process the process to be removed.
   */
  void removeProcess(Process process);

  /**
   * Elects a new backup process. Returns <code>null</code> if no process is
   * available for being elected.
   * 
   * @return the elected backup, <code>null</code> if no process is available.
   * @throws InterruptedException if the component is interrupted while waiting
   *           for a process to be added in the group.
   */
  Process electBackup() throws InterruptedException;

  /**
   * Elects the backup as new leader provided the curerntly registered leader is
   * the one given in parameter. Otherwise, don't do anything.
   * 
   * @param oldLeader the process that were supposed to be the old leader.
   * @return the new leader.
   */
  Process electBackupAsLeader(Process oldLeader);

}