/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.pump;

import org.objectweb.dream.PullException;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;

/**
 * Extention of the basic pump that use a period <code>T</code> (in ms). More
 * precisely, every T ms, a message is pulled on the input and is then pushed on
 * the output. If these two actions (pull then push) are longer than T ms, an
 * other message is pulled immediately.
 */
public class PumpPeriodicSynchronizedImpl extends PumpSynchronizedImpl
    implements
      PumpPeriodicAttributeController
{

  /**
   * The initialization parameter that can be passed to this component to
   * specify the pump's period.
   */
  public static final String PERIOD_INIT_PARAM_NAME = "period";

  protected long             period                 = -1;

  /**
   * Default constructor
   */
  public PumpPeriodicSynchronizedImpl()
  {
    pumpTask = new PeriodicPumpTask();
  }

  class PeriodicPumpTask extends PumpTask
  {

    /**
     * @see org.objectweb.dream.control.activity.task.Task#execute(Object)
     */
    public Object execute(Object hints) throws InterruptedException
    {

      long begin = System.currentTimeMillis();
      if (mutexItf != null)
      {
        mutexItf.lock();
      }
      try
      {
        Message msg = inPullItf.pull(null);
        if (msg != null || pushNullPolicy)
        {
          outPushItf.push(msg, null);
        }
      }
      catch (PullException e)
      {
        throw new InterruptedException(e.getLocalizedMessage());
      }
      catch (PushException e)
      {
        throw new InterruptedException(e.getLocalizedMessage());
      }
      finally
      {
        if (mutexItf != null)
        {
          mutexItf.unlock();
        }
      }

      long sleepTime = period - (System.currentTimeMillis() - begin);
      if (sleepTime > 0)
      {
        Thread.sleep(sleepTime);
      }
      return EXECUTE_AGAIN;
    }
  }

  /**
   * @see org.objectweb.dream.pump.PumpPeriodicAttributeController#getPeriod()
   */
  public long getPeriod()
  {
    return period;
  }

  /**
   * @see org.objectweb.dream.pump.PumpPeriodicAttributeController#setPeriod(long)
   */
  public void setPeriod(long period)
  {
    if (period < 0)
    {
      throw new IllegalArgumentException("Period value negative");
    }
    this.period = period;
  }
}