/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Matthieu Leclercq
 */

package org.objectweb.dream.queue;

import java.util.Collections;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.control.activity.Util;
import org.objectweb.dream.control.activity.task.AbstractTask;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Basic implementation of a Pull outgoing handler.
 */
public class PushOutgoingHandlerActiveImpl extends AbstractComponent

{

  // Client interfaces
  protected MessageManager        messageManagerItf;
  protected Push                  outPushItf;
  protected Buffer                bufferItf;
  protected BufferRemoveFirstLast bufferRemoveFirstLastItf;

  // The task
  protected Task                  queueTask = new PushPushQueueTask();

  // -------------------------------------------------------------------------
  // Overriden methods
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.AbstractComponent#beforeFirstStart(org.objectweb.fractal.api.Component)
   */
  protected void beforeFirstStart(Component componentItf)
      throws IllegalLifeCycleException
  {
    try
    {
      Util.addTask(componentItf, queueTask, Collections.EMPTY_MAP);
    }
    catch (Exception e)
    {
      throw new IllegalLifeCycleException("Can't add task");
    }
  }

  class PushPushQueueTask extends AbstractTask
  {

    /**
     * default constructor
     */
    public PushPushQueueTask()
    {
      super("PushPushQueueTask");
    }

    /**
     * @see Task#execute(Object)
     */
    public Object execute(Object hints) throws InterruptedException
    {
      Message msg = bufferItf.remove();
      try
      {
        outPushItf.push(msg, null);
      }
      catch (PushException e)
      {
        handlePushException(msg, e);
      }
      return EXECUTE_AGAIN;
    }
  }

  // ---------------------------------------------------------------------------
  // Utility methods.
  // ---------------------------------------------------------------------------

  /**
   * Handles a {@link PushException}occuring when the activity try to push a
   * message. This implementation simlpy delete message using message manger.
   * 
   * @param message the pushed message
   * @param exception the exceptino thrown during the push call.
   */
  protected void handlePushException(Message message, PushException exception)
  {
    logger.log(BasicLevel.WARN, "A Push exception has been cached", exception);
    messageManagerItf.deleteMessage(message);
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Push.OUT_PUSH_ITF_NAME))
    {
      outPushItf = (Push) serverItf;
    }
    else if (clientItfName.equals(Buffer.ITF_NAME))
    {
      bufferItf = (Buffer) serverItf;
    }
    else if (clientItfName.equals(BufferRemoveFirstLast.ITF_NAME))
    {
      bufferRemoveFirstLastItf = (BufferRemoveFirstLast) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Push.OUT_PUSH_ITF_NAME, Buffer.ITF_NAME,
        BufferRemoveFirstLast.ITF_NAME, MessageManager.ITF_NAME};
  }

}