/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.router;

import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * A basic router implementation with two outputs :
 * <code>out-push-with-chunk</code> and <code>out-push-without-chunk</code>.
 * The first one is used if pushed messages contain a specified chunk name, the
 * secone one is used otherwise. The chunk name to test is specified as
 * attribute. <br>
 * <b>Warning </b>: This router may not work correctly if it is used with
 * unextensible message implementations that simply return <code>this</code>
 * in the {@link Message#getChunk(String) }method. In this case messages will
 * always be routed through the <code>out-push-with-chunk</code> output.
 */
public class RouterChunkNameImpl extends AbstractComponent
    implements
      Push,
      RouterChunkNameAttributeController
{

  /**
   * The name of client interface used to push messages with the specified chunk
   * name.
   */
  public static final String OUTPUSH_WITH_CHUNK_ITF_NAME    = "out-push-with-chunk";

  /**
   * The name of client interface used to push messages without the specified
   * chunk name.
   */
  public static final String OUTPUSH_WITHOUT_CHUNK_ITF_NAME = "out-push-without-chunk";

  /** The interface used to push messages with the specified chunk name. */
  protected Push             outPushWithChunkItf;
  /** The interface used to push messages without the specified chunk name. */
  protected Push             outPushWithoutChunkItf;
  /** the chunk name to test */
  protected String           chunkName;

  /**
   * @see Push#push(Message, Map)
   */
  public void push(Message message, Map context) throws PushException
  {
    if (message.getChunk(chunkName) == null)
    {
      outPushWithoutChunkItf.push(message, context);
    }
    else
    {
      outPushWithChunkItf.push(message, context);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of RouterChunkNameAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see RouterChunkNameAttributeController#getChunkName()
   */
  public String getChunkName()
  {
    return chunkName;
  }

  /**
   * @see RouterChunkNameAttributeController#setChunkName(String)
   */
  public void setChunkName(String chunkName)
  {
    this.chunkName = chunkName;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{OUTPUSH_WITH_CHUNK_ITF_NAME,
        OUTPUSH_WITHOUT_CHUNK_ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(OUTPUSH_WITH_CHUNK_ITF_NAME))
    {
      outPushWithChunkItf = (Push) serverItf;
    }
    else if (clientItfName.equals(OUTPUSH_WITHOUT_CHUNK_ITF_NAME))
    {
      outPushWithoutChunkItf = (Push) serverItf;
    }
  }
}